"""
Canonical home for Jena sidecar interface types and utilities.

Modules:
  - jena_types: Python dataclasses mirroring Jena sidecar JSON types.
  - jena_ast_mapper: JSON → Python Op tree mapper.
  - jena_sidecar_client: HTTP client for the Jena SPARQL compiler sidecar.
"""
